#!/usr/bin/env python

"""
SMI module for python. This is merely a wrapper for the libsmi library
(written in C).  This is the user-visible interface to the libsmi wrapper.
This module subclasses the SWIG generated shadow classes and adds some
convenience methods and provides a more object oriented interface to the
libsmi C library.

You need SWIG 1.3a5 plus my SWIG patch to generate the libsmi modules. 

Keith Dart <kdart@cosinecom.com>

"""

from UserList import UserList
from UserDict import UserDict
import string

# this is the SWIG generated shadow class file.
from libsmi import *

from SNMP import OID

SmiError = libsmic.SmiError

BASETYPEMAP = {
	SMI_BASETYPE_UNKNOWN           : "<UNKNOWN>" ,
	SMI_BASETYPE_OCTETSTRING       : "OctetString" ,
	SMI_BASETYPE_OBJECTIDENTIFIER  : "ObjectIdentifier" ,
	SMI_BASETYPE_UNSIGNED32        : "Unsigned32" ,
	SMI_BASETYPE_INTEGER32         : "Integer32" ,
	SMI_BASETYPE_UNSIGNED64        : "Unsigned64" ,
	SMI_BASETYPE_INTEGER64         : "Integer64" ,
	SMI_BASETYPE_FLOAT32           : "Float32" ,
	SMI_BASETYPE_FLOAT64           : "Float64" ,
	SMI_BASETYPE_FLOAT128          : "Float128" ,
	SMI_BASETYPE_ENUM              : "Enumeration" ,
	SMI_BASETYPE_BITS              : "Bits" ,
}
    

# helper classes

class Index(UserList):
	def __init__(self, namelist=None, implied=0):
		UserList.__init__(self, namelist)
		# only the last index node can be IMPLIED, so the IMPLIED flag
		# applies only to the last element (Index[-1]).
		self.implied = implied

	def __repr__(self):
		return "%s(namelist=%s, implied=%d)" % (self.__class__.__name__, repr(self.data), self.implied)

class Enumeration(UserDict):
	def __getattr__(self, key):
		if self.data.has_key(key):
			return self.data[key]
		else:
			raise AttributeError, "Enumeration: no such enumeration"
	def __repr__(self):
		return "%s(%s)" % (self.__class__.__name__, repr(self.data))
	def __str__(self):
		s = ["{ "]
		for key, val in self.data.items():
			s.append("%s(%d) " % (key, val))
		s.append("}")
		return string.join(s, "")

### libsmi derived classes. the "Smi" prefix is dropped here. After all,
# these do live in the SMI name space.

class Value(SmiValue):
	def __str__(self):
		if self.basetype == SMI_BASETYPE_UNKNOWN:
			return "<unknown>"
		elif self.basetype == SMI_BASETYPE_INTEGER32 or \
							self.basetype == SMI_BASETYPE_UNSIGNED32:
			return str(self.value.integer32)
		elif self.basetype == SMI_BASETYPE_FLOAT32 or \
							self.basetype == SMI_BASETYPE_FLOAT64:
			return str(self.value.float32)
		elif self.basetype == SMI_BASETYPE_OCTETSTRING:
			s = []
			for i in xrange(self.len):
				s.append(repr(ptrvalue(self.value.ptr, i, "char")))
			return string.join(s, "")
		elif self.basetype == SMI_BASETYPE_OBJECTIDENTIFIER:
			return str(List_FromSmiSubid(self.value.oid, self.len))
		elif self.basetype == SMI_BASETYPE_ENUM: # FIXME
			return "Value: Enumeration..."
		elif self.basetype == SMI_BASETYPE_BITS:
			return "Value: BITS..."
# not implemented
#    SMI_BASETYPE_FLOAT128
#    SMI_BASETYPE_INTEGER64
#    SMI_BASETYPE_UNSIGNED64

class Type(SmiType):
	def __str__(self):
		if self.name:
			return self.name
		else:
			return BASETYPEMAP[self.basetype]
	def get_attributes(self):
		return self.__getmethods__.keys()
	def getModule(self):
		return smiGetTypeModule(self.this)
	def getNamedNumbers(self):
		nnlist = []
		if self.basetype == SMI_BASETYPE_ENUM or self.basetype == SMI_BASETYPE_BITS:
			nn = smiGetFirstNamedNumber(self.this)
			while nn:
				nnlist.append(NamedNumber(nn.this))
				nn = smiGetNextNamedNumber(nn.this)
		return nnlist
	def getEnumeration(self):
		nnlist = self.getNamedNumbers()
		if nnlist:
			enum = Enumeration()
			for nn in nnlist:
				enum[nn.name] = nn.value.value.integer32
			return enum
		else:
			return None

	def getRanges(self):
		nnlist = []
		nn = smiGetFirstRange(self.this)
		while nn:
			nnlist.append(Range(nn.this))
			nn = smiGetNextRange(nn.this)
		return nnlist

SmiType.__getmethods__["namednumbers"] = Type.getNamedNumbers
SmiType.__getmethods__["enumeration"] = Type.getEnumeration

class NamedNumber(SmiNamedNumber):
	def __str__(self):
		return "%s(%d)" % (self.name, self.value.value.integer32)
	def __cmp__(self, other):
		return cmp(self.value.value.integer32, other.value.value.integer32)
	def __int__(self):
		return self.value.value.integer32
	def __long__(self):
		return long(self.value.value.integer32)


class Range(SmiRange):
	def __str__(self):
		return "(%d .. %d)" % (self.minValue.value.integer32, self.maxValue.value.integer32)
	def getMinValue(self):
		return self.minValue.value.integer32
	def getMaxValue(self):
		return self.maxValue.value.integer32

class Import(SmiImport):
	def __str__(self):
		return "%s(module=%s, name=%s)" % (self.__class__.__name__, self.module, self.name)

class Option(SmiOption):
	def __str__(self):
		return self.description
	def getNode(self):
		n = smiGetOptionNode(self.this)
		if n:
			return Node(n.this)
		else:
			return None

class Refinement(SmiRefinement):
	pass

class Element(SmiElement):
	def getNode(self):
		n = smiGetElementNode(self.this)
		if n:
			return Node(n.this)
		else:
			return None

class Revision(SmiRevision):
	pass

class Macro(SmiMacro):
	def getModule(self):
		return Module(smiGetMacroModule(self.this).this)


class Node(SmiNode):
	def __str__(self):
		s = [self.__class__.__name__+":"]
		for key, meth in self.__getmethods__.items():
			s.append("   %20s = %s" % (key, meth(self)))
		return string.join(s, "\n")

	def get_attributes(self):
		return self.__getmethods__.keys()

	def isWritable(self):
		return self.access >= SMI_ACCESS_READ_WRITE
	
	def isReadable(self):
		return self.access >= SMI_ACCESS_READ_ONLY

	def isNotAccessible(self):
		return self.access == SMI_ACCESS_NOT_ACCESSIBLE

	def getChildren(self, statusfilt=None):
		nodes = []
		fn = smiGetFirstChildNode(self.this)
		while fn:
			nodes.append(Node(fn.this))
			fn = smiGetNextChildNode(fn.this)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes

	def getOptions(self):
		nodes = []
		fn = smiGetFirstOption(self.this)
		while fn:
			nodes.append(Option(fn.this))
			fn = smiGetNextOption(fn.this)
		return nodes

	def getRefinements(self, accessfilt=None):
		nodes = []
		fn = smiGetFirstRefinement(self.this)
		while fn:
			nodes.append(Refinement(fn.this))
			fn = smiGetNextRefinement(fn.this)
		if accessfilt:
			return filter(lambda n, sf=accessfilt: n.access == sf, nodes)
		else:
			return nodes

	def getType(self):
		t = smiGetNodeType(self.this)
		if t:
			return Type(t.this)
		else:
			return None

	def getOID(self):
		return OID(List_FromSmiSubid(self.oid, self.oidlen))

	def _getElements(self, node):
		il = []
		el = smiGetFirstElement(node.this)
		while el:
			il.append(el.this)
			el = smiGetNextElement(el.this)
		return il

	def getIndex(self):
		if self.nodekind == SMI_NODEKIND_ROW:
			indexes = Index()
			if self.indexkind == SMI_INDEX_INDEX:
				il = self._getElements(self)
				if il:
					for el in il:
						n = smiGetElementNode(el)
						indexes.append(n.name)
					indexes.implied = self.implied
				return indexes
			elif self.indexkind == SMI_INDEX_AUGMENT:
				augnode = smiGetRelatedNode(self.this)
				il = self._getElements(augnode)
				if il:
					for el in il:
						n = smiGetElementNode(el)
						indexes.append(n.name)
					indexes.implied = augnode.implied
				return indexes
			else:
				raise SmiError, "Node.getIndex: can't handle this indexkind"
		else:
			return None

# patch the Node to map retreival methods to extra attributes.
SmiNode.__getmethods__["OID"] = Node.getOID
SmiNode.__getmethods__["syntax"] = Node.getType
SmiNode.__getmethods__["index"] = Node.getIndex

class Module(SmiModule):
	def __str__(self):
		s = [self.__class__.__name__+":"]
		for key, meth in self.__getmethods__.items():
			s.append("   %20s = %s" % (key, meth(self)))
		return string.join(s, "\n")

	def getPtr(self):
		return self.this

	def get_attributes(self):
		return self.__getmethods__.keys()

	def getImports(self):
		imports = []
		fn = smiGetFirstImport(self.this)
		while fn:
			imports.append(Import(fn.this))
			fn = smiGetNextImport(fn.this)
		return imports

	def getType(self, name):
		t = smiGetType(self.this, name)
		if t:
			return Type(t.this)
		else:
			return None

	def getTypes(self, statusfilt=None):
		nodes = []
		ft = smiGetFirstType(self.this)
		while ft:
			nodes.append(Type(ft.this))
			ft = smiGetNextType(ft.this)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes

	def getRevisions(self):
		nodes = []
		ft = smiGetFirstRevision(self.this)
		while ft:
			nodes.append(Revision(ft.this))
			ft = smiGetNextRevision(ft.this)
		return nodes

	def isImported(self, module, name):
		return smiIsImported(self.this, module.this, name)

	def getMacro(self, name):
		m = smiGetMacro(self.this, name)
		if m:
			return Macro(m.this)
		else:
			return None

	def getMacros(self, statusfilt=None):
		nodes = []
		fn = smiGetFirstMacro(self.this)
		while fn:
			nodes.append(Macro(fn.this))
			fn = smiGetNextMacro(fn.this, kind)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes

	def getNode(self, name):
		n = smiGetNode(self.this, name)
		if n:
			return Node(n.this)
		else:
			return None
	
	def getNodes(self, kind=SMI_NODEKIND_ANY):
		nodes = []
		fn = smiGetFirstNode(self.this, kind)
		while fn:
			nodes.append(Node(fn.this))
			fn = smiGetNextNode(fn.this, kind)
		return nodes
		
		
	def getNodeByOID(self, oid):
		oid = list(oid)
		val = GetNodeByOID(oid)
		if val: val = Node(val)
		return val


	def getScalars(self, statusfilt=None):
		nodes = self.getNodes(SMI_NODEKIND_SCALAR)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes

	def getRows(self, statusfilt=None):
		nodes = self.getNodes(SMI_NODEKIND_ROW)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes

	def getNotifications(self, statusfilt=None):
		nodes = self.getNodes(SMI_NODEKIND_NOTIFICATION)
		if statusfilt:
			return filter(lambda n, sf=statusfilt: n.status == sf, nodes)
		else:
			return nodes


### factory functions. Start here.

def getModule(modulename):
	smiLoadModule(modulename)
	m = smiGetModule(modulename)
	if m:
		return Module(m.this)

def getModules():
	mods = []
	m = smiGetFirstModule()
	while m:
		mods.append(Module(m.this))
		m = smiGetNextModule(m.this)
	return mods

loadModule = smiLoadModule

def loadModules(*args):
	"""
loadModules(modname|modnamelist, ...)
Takes multiple string arguments, or lists of strings, which should be
module names, and loads them.

	"""
	for arg in args:
		if type(arg) is type([]):
			for argl in arg:
				loadModule(argl)
		else:
			loadModule(arg)

### MAIN initialisation
# pre-load commonly used modules
loadModules("SNMPv2-SMI", "SNMPv2-TC", "SNMPv2-CONF", "SNMPv2-MIB")

